#!/bin/bash
#--------------------------------------------------------------------------
#
# java-launcher
#
#--------------------------------------------------------------------------
#
# Author: (c) 2024 - Invoca Systems <simon.matter@invoca.ch>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by the
# Free Software Foundation, either version 2 of the license or, at your
# option, any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.
#
#--------------------------------------------------------------------------

# Treat unset variables and parameters other than the special
# parameters `@' and `*' as an error when performing parameter expansion.
set -o nounset

# Find our real script name
if ! (( ${JAVA_LAUNCHER+1} )); then
  JAVA_LAUNCHER="$(readlink -e "$0")"
  JAVA_LAUNCHER="${JAVA_LAUNCHER##*/}"
else
  echo "Error: \`\$JAVA_LAUNCHER' is already set, exiting!" >&2
  exit 1
fi

# Show usage and exit
usage_exit() {
  cat << EOF >&2
$JAVA_LAUNCHER version 0.2.3
Copyright (c) 2024 - Invoca Systems
This program may be freely redistributed under the terms of the GNU GPL

Java-launcher helps you to manage multiple Java versions installed as part
of the OS (managed by \`alternatives') and also Java installations in
/opt/java/default/ or elsewhere.

By default it runs a Java binary in \`/opt/java/default/bin/' if it is
available and sets \`\$JAVA_HOME' accordingly on the fly.

To run a non default Java version, use one of the methods below:

  * run \`JAVA_HOME="/opt/java/jdk-22.0.1" java -version'

  * add \`JAVA_HOME="/opt/java/jdk-22.0.1"' to \`\${HOME}/.java-launcher'

EOF
  if [[ "$2" ]]; then
    echo "$2" >&2
  fi
  exit $1
}

# Do not run this script directly by its own name but only to show help
if [[ "${0##*/}" == "$JAVA_LAUNCHER" ]]; then
  if (( ${1+1} )) && [[ "$1" == "-h" ]]; then
    usage_exit 0 ""
  else
    usage_exit 1 "Error: do not run \`${JAVA_LAUNCHER}' directly by its own name, exiting!"
  fi
fi

# Read custom configuration from `${HOME}/.java-launcher' if `$JAVA_HOME' is not set
! (( ${JAVA_HOME+1} )) &&
  (( ${HOME+1} )) &&
  [[ "$HOME" ]] &&
  [[ -r "${HOME}/.java-launcher" ]] &&
  . "${HOME}/.java-launcher"

# Set default `$JAVA_HOME' if it is available and `$JAVA_HOME' is still not set
! (( ${JAVA_HOME+1} )) &&
  [[ -x "/opt/java/default/bin/java" ]] &&
  JAVA_HOME="/opt/java/default"

# Run the target binary if it exists in `${JAVA_HOME}/bin/' and is executable
(( ${JAVA_HOME+1} )) &&
  [[ "$JAVA_HOME" ]] &&
  [[ -x "${JAVA_HOME}/bin/${0##*/}" ]] &&
  JAVA_HOME="$(readlink -e "${JAVA_HOME}")" &&
  export JAVA_HOME &&
  unset -v JAVA_LAUNCHER &&
  exec "${JAVA_HOME}/bin/${0##*/}" "$@"

# Fallback to Java provided by the OS if `$JAVA_HOME' is still not set
! (( ${JAVA_HOME+1} )) &&
  [[ -x "/usr/bin/${0##*/}" ]] &&
  unset -v JAVA_LAUNCHER &&
  exec "/usr/bin/${0##*/}" "$@"

# If we came so far it means the target binary was not usable
usage_exit 127 "Error in \`${JAVA_LAUNCHER}': no usable target binary \`${0##*/}' found, exiting!"
